import 'dotenv/config';
import { Injectable, Logger, Module, OnModuleDestroy, OnModuleInit } from '@nestjs/common';
import { AppController } from './app.controller';
import { AppService } from './app.service';
import { GraphQLModule } from '@nestjs/graphql';
import { ApolloDriver, ApolloDriverConfig } from '@nestjs/apollo';
import { join } from 'path';
import { MemberResolver } from './services/reputation/components/member/member.resolver';
import { PostResolver } from './services/reputation/components/post/post.resolver';
import { CitationsResolver } from './services/reputation/components/citation/citation.resolver';

import { DatabaseConnector } from './util/database-connector';
import { MemberRepository } from './services/reputation/components/member/member.repository';
const dbConnector = new DatabaseConnector(new Logger('DatabaseConnector'));

@Injectable()
export class StartupService implements OnModuleInit {
  async onModuleInit() {

    const testSuite = process.env.TEST_SUITE;
    if (testSuite?.toLowerCase().includes("integration")) return;

    const host = process.env.DB_HOST;
    const port = process.env.DB_PORT;
    const user = process.env.DB_USER;
    const pass = process.env.DB_PASS;

    if (!host) throw new Error('Missing environment variable, DB_HOST');
    if (!port) throw new Error('Missing environment variable, DB_PORT');
    if (!user) throw new Error('Missing environment variable, DB_USER');
    if (!pass) throw new Error('Missing environment variable, DB_PASSWORD');

    const options = {host, port, user, pass};
    await dbConnector.connect(options, [
      MemberRepository
    ]);
  }
}

@Injectable()
export class ShutdownService implements OnModuleDestroy {
  async onModuleDestroy() {
    await dbConnector.disconnect();
  }
}

@Module({
  imports: [
    GraphQLModule.forRoot<ApolloDriverConfig>({
      driver: ApolloDriver,
      typePaths: ['./**/*.graphql'],
      definitions: {
        path: join(process.cwd(), 'src/graphql.ts')
      },
    }),
  ],
  controllers: [AppController],
  providers: [
    AppService,
    StartupService,
    ShutdownService,
    MemberResolver,
    PostResolver,
    CitationsResolver
  ],
})

export class AppModule {}
